@@ -49,6 +49,7 @@ from hypothesis.errors import (
     DidNotReproduce,
     FailedHealthCheck,
     Flaky,
+    Found,
     HypothesisDeprecationWarning,
     HypothesisWarning,
     InvalidArgument,
@@ -65,12 +66,11 @@ from hypothesis.internal.compat import (
     binary_type,
     get_type_hints,
     getfullargspec,
-    hbytes,
     int_from_bytes,
     qualname,
 )
 from hypothesis.internal.conjecture.data import ConjectureData, StopTest
-from hypothesis.internal.conjecture.engine import ConjectureRunner, ExitReason, sort_key
+from hypothesis.internal.conjecture.engine import ConjectureRunner, sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
     escalate_hypothesis_internal_error,
@@ -85,7 +85,6 @@ from hypothesis.internal.reflection import (
     get_pretty_function_description,
     impersonate,
     is_mock,
-    nicerepr,
     proxies,
 )
 from hypothesis.reporting import (
@@ -111,10 +110,6 @@ running_under_pytest = False
 global_force_seed = None
 
 
-def new_random():
-    return rnd_module.Random(rnd_module.getrandbits(128))
-
-
 @attr.s()
 class Example(object):
     args = attr.ib()
@@ -490,7 +485,9 @@ def new_given_argspec(original_argspec, given_kwargs):
 
 
 class StateForActualGivenExecution(object):
-    def __init__(self, test_runner, search_strategy, test, settings, random, had_seed):
+    def __init__(
+        self, test_runner, search_strategy, test, settings, random, wrapped_test,
+    ):
         self.test_runner = test_runner
         self.search_strategy = search_strategy
         self.settings = settings
@@ -500,7 +497,10 @@ class StateForActualGivenExecution(object):
         self.random = random
         self.__warned_deadline = False
         self.__test_runtime = None
-        self.__had_seed = had_seed
+
+        self.__had_seed = wrapped_test._hypothesis_internal_use_seed
+        self.is_find = getattr(wrapped_test, "_hypothesis_internal_is_find", False)
+        self.wrapped_test = wrapped_test
 
         self.test = test
 
@@ -520,6 +520,9 @@ class StateForActualGivenExecution(object):
         it might have placed ``data`` in an unsuccessful state and then
         swallowed the corresponding control exception.
         """
+
+        data.is_find = self.is_find
+
         text_repr = [None]
         if self.settings.deadline is None:
             test = self.test
@@ -675,10 +678,14 @@ class StateForActualGivenExecution(object):
         """
         # Tell pytest to omit the body of this function from tracebacks
         __tracebackhide__ = True
-        if global_force_seed is None:
-            database_key = function_digest(self.test)
-        else:
-            database_key = None
+        try:
+            database_key = self.wrapped_test._hypothesis_internal_database_key
+        except AttributeError:
+            if global_force_seed is None:
+                database_key = function_digest(self.test)
+            else:
+                database_key = None
+
         runner = ConjectureRunner(
             self._execute_once_for_engine,
             settings=self.settings,
@@ -727,7 +734,7 @@ class StateForActualGivenExecution(object):
             try:
                 self.execute_once(
                     ran_example,
-                    print_example=True,
+                    print_example=not self.is_find,
                     is_final=True,
                     expected_failure=(
                         info.__expected_exception,
@@ -942,12 +949,7 @@ def given(
                 )
 
             state = StateForActualGivenExecution(
-                test_runner,
-                search_strategy,
-                test,
-                settings,
-                random,
-                had_seed=wrapped_test._hypothesis_internal_use_seed,
+                test_runner, search_strategy, test, settings, random, wrapped_test,
             )
 
             reproduce_failure = wrapped_test._hypothesis_internal_use_reproduce_failure
@@ -1101,7 +1103,9 @@ def find(
     )
     if settings is None:
         settings = Settings(max_examples=2000)
-    settings = Settings(settings, suppress_health_check=HealthCheck.all())
+    settings = Settings(
+        settings, suppress_health_check=HealthCheck.all(), report_multiple_bugs=False
+    )
 
     if database_key is None and settings.database is not None:
         database_key = function_digest(condition)
@@ -1113,61 +1117,24 @@ def find(
         )
     specifier.validate()
 
-    search = specifier
+    last = [None]
 
-    random = random or new_random()
-    successful_examples = [0]
-    last_data = [None]
-    last_repr = [None]
+    @settings
+    @given(specifier)
+    def test(v):
+        if condition(v):
+            last[0] = v
+            raise Found()
 
-    def template_condition(data):
-        with deterministic_PRNG():
-            with BuildContext(data):
-                try:
-                    data.is_find = True
-                    result = data.draw(search)
-                    data.note(result)
-                    success = condition(result)
-                except UnsatisfiedAssumption:
-                    data.mark_invalid()
+    if random is not None:
+        test = seed(random.getrandbits(64))(test)
 
-        if success:
-            successful_examples[0] += 1
+    test._hypothesis_internal_is_find = True
+    test._hypothesis_internal_database_key = database_key
 
-        if settings.verbosity >= Verbosity.verbose:
-            if not successful_examples[0]:
-                report(u"Tried non-satisfying example %s" % (nicerepr(result),))
-            elif success:
-                if successful_examples[0] == 1:
-                    last_repr[0] = nicerepr(result)
-                    report(u"Found satisfying example %s" % (last_repr[0],))
-                    last_data[0] = data
-                elif (
-                    sort_key(hbytes(data.buffer)) < sort_key(last_data[0].buffer)
-                ) and nicerepr(result) != last_repr[0]:
-                    last_repr[0] = nicerepr(result)
-                    report(u"Shrunk example to %s" % (last_repr[0],))
-                    last_data[0] = data
-        if success and not data.frozen:
-            data.mark_interesting()
-
-    runner = ConjectureRunner(
-        template_condition, settings=settings, random=random, database_key=database_key
-    )
-
-    runner.run()
-    note_engine_for_statistics(runner)
-    if runner.interesting_examples:
-        data = ConjectureData.for_buffer(
-            list(runner.interesting_examples.values())[0].buffer
-        )
-        with deterministic_PRNG():
-            with BuildContext(data):
-                return data.draw(search)
-    if runner.valid_examples == 0 and (runner.exit_reason != ExitReason.finished):
-        raise Unsatisfiable(
-            "Unable to satisfy assumptions of %s."
-            % (get_pretty_function_description(condition),)
-        )
+    try:
+        test()
+    except Found:
+        return last[0]
 
     raise NoSuchExample(get_pretty_function_description(condition))
